
class GameObject:
    """
    GameObject has:
    x,y coordinates, positive integers, zero point is at upper left corner
    x, y sizes as positive integers, object rectangle is defined by x +/- x_size and  y +/- y_size
    collision detection/reaction
    """

    def __init__(self, x, y, x_size, y_size, obj_type="default"):   # __init__ tekitab klassi eksemplari,
                                                # väljakutse mo=MovingObject(x,y,x_s,y_s)
                                                # NB! Väljakutsel jääb self argumendina ära 
        """ Initialize coordinates and sizes"""
        self.x = x # Tekitab ja väärtustab meie objektil atribuudi x
        self.y = y
        self.x_size = x_size
        self.y_size = y_size
        self.obj_type = obj_type

    def collides(self, other):
        """ Detect if our size rect. overlaps with other GameObjects """
        x_overlaps = abs(self.x - other.x) < self.x_size + other.x_size
        y_overlaps = abs(self.y - other.y) < self.y_size + other.y_size
        return x_overlaps and y_overlaps

    def collision_react(self, other):
        """ React to collision with other GameObject """
        pass


class MovingObject(GameObject): # Kuna MovingObject on GameObjecti alamklass, siis
                                # võtab see kõik GameObjecti meetodid vaikimisi üle.                
    """
    MovingObject is a GameObject with additional movement methods and heading (dx, dy)
    """

    def __init__(self, x, y, x_size, y_size, dx, dy, obj_type="default"):
        """ dx, dy: heading, change of x,y coordinates """
        super().__init__(x, y, x_size, y_size, obj_type) # Ülemklassi meetodi väljakutse
        self.dx = dx
        self.dy = dy

    def set_heading(self, dx, dy):
        self.dx = dx
        self.dy = dy

    def tick(self):
        """ React to timer tick by moving """
        self.move(self.dx, self.dy)

    def move(self, dx=0, dy=0):
        """ Move the object by dx, dy """
        self.x += dx
        self.y += dy

        
class PacMan(MovingObject): # MovingObject on PacMani ülemklass
    """
    Represents the PacMan moving object.
    """
    pass


if __name__ == "__main__":
    mo1 = GameObject(100, 100, 10, 10)
    mo2 = MovingObject(115, 115, 10, 10, 0, -10)
    print(mo1.collides(mo2), mo2.collides(mo1))
    mo3 = GameObject(115, 215, 10, 10)
    print(mo3.collides(mo2), mo2.collides(mo3))
    for step in range(14):
        mo2.tick()
        mo2.set_heading(0, 10)
        print("step", step,  mo2.x, mo2.y, mo3.collides(mo2), mo2.collides(mo3))
    
        
